@@ -186,19 +186,23 @@ Default Parameters
 
 Copyright (C) 2013 CERN
 """
+import logging
+import os
 import re
-from simplevisor.errors import SimplevisorError
-import simplevisor.log as log
 import sys
 import time
-from simplevisor import utils
-from simplevisor.config_utils import mutex, reqall, reqany
-try:
-    from urllib import unquote
-except ImportError:
-    from urllib.parse import unquote
 
-MAXIMUM_LOG = 100
+from mtb.modules import md5_hash, unquote
+from mtb.proc import \
+    timed_process, ProcessTimedout, ProcessError, \
+    kill_pids, merge_status, pidof, which
+from mtb.validation import mutex, reqall, reqany, get_int_or_die
+
+from simplevisor.errors import ServiceError
+
+LOGGER = logging.getLogger("simplevisor")
+
+MAX_LOG_MESSAGES = 100
 DEFAULT_TIMEOUT = 60
 DEFAULT_EXPECTED = "running"
 
@@ -215,60 +219,64 @@ class Service(object):
                  **kwargs):
         """ Service constructor. """
         self.name = name
-        self._opts = {"name": name,
-                      "expected": expected,
-                      "control": control,
-                      "daemon": daemon,
-                      "path": path,
-                      "pattern": pattern,
-                      "restart": restart,
-                      "start": start,
-                      "status": status,
-                      "stop": stop,
-                      "timeout": utils.get_int_or_die(
-                          timeout,
-                          "timeout value for %s is not a valid integer: %s" %
-                          (name, timeout)), }
-        self._status = {"name": name,
-                        "log": list(),
-                        }
-        for key in kwargs.keys():
+        self._opts = {
+            "name": name,
+            "expected": expected,
+            "control": control,
+            "daemon": daemon,
+            "path": path,
+            "pattern": pattern,
+            "restart": restart,
+            "start": start,
+            "status": status,
+            "stop": stop,
+            "timeout": get_int_or_die(
+                timeout,
+                "timeout value for %s is not a valid integer: %s" %
+                (name, timeout)), }
+        self._status = {
+            "name": name,
+            "log": list(), }
+        for key in kwargs:
             if not key.startswith("var_"):
-                raise SimplevisorError(
-                    "an invalid property has been specified for %s: %s" %
-                    (name, key))
+                raise ValueError(
+                    "invalid property for service %s: %s" % (name, key))
         self._is_new = True
         try:
             self._validate_opts()
         except ValueError:
             error = sys.exc_info()[1]
-            raise SimplevisorError(
-                "Service %s configuration error: %s" % (self.name, error))
+            raise ValueError(
+                "service %s is not properly configured: %s" %
+                (self.name, error))
         if control is None and daemon is not None:
+            si_loop = which("simplevisor-loop")
+            if si_loop is None:
+                raise RuntimeError(
+                    "cannot find simplevisor-loop command in the "
+                    "environment: %s" % (os.environ["PATH"], ))
+            common_path = "%s --pidfile %s" % (si_loop, daemon, )
             self._opts["start"] = (
-                "/usr/bin/simplevisor-loop -c 1 "
-                "--pidfile %s --daemon %s" % (daemon, start))
-            self._opts["stop"] = (
-                "/usr/bin/simplevisor-loop --pidfile %s --quit" % (daemon, ))
-            self._opts["status"] = (
-                "/usr/bin/simplevisor-loop --pidfile %s --status" % (daemon, ))
+                "%s -c 1 --daemon %s" % (common_path, start))
+            self._opts["stop"] = ("%s --quit" % (common_path, ))
+            self._opts["status"] = ("%s --status" % (common_path, ))
         elif control is None and status is None:
             if sys.platform != "linux2":
-                msg = "don't know how to read process table, you " + \
-                      "must specify a status command for service %s" % name
-                raise ValueError(msg)
+                raise ValueError(
+                    "don't know how to read process table, you must specify "
+                    "a status command for service: %s" % (name, ))
             if pattern is not None:
                 pat = pattern
             else:
                 pat = " ".join(self.get_cmd("start"))
             try:
                 self._opts["pattern_re"] = re.compile(pat)
-                log.LOG.debug("using %s as pattern for service %s" %
-                              (pat, name))
+                LOGGER.debug(
+                    "using %s as pattern for service %s" % (pat, name))
             except re.error:
                 error = sys.exc_info()[1]
-                msg = "%s service pattern not valid: %s" % error
-                raise ValueError(msg)
+                raise ValueError(
+                    "%s service pattern not valid: %s" % (name, error))
 
     def _validate_opts(self):
         """ Validate options. """
@@ -289,97 +297,213 @@ class Service(object):
     def get_child(self, path):
         """
         Return a child by its path.
+        :param path: list containing the tokens representing the path to the
+        requested child
         """
         first = path.pop(0)
         if first == self.name:
             return self
         raise ValueError("not found")
 
-    def get_cmd(self, subcmd):
+    def get_cmd(self, sub_command):
         """
         Return a command given the sub command.
+        :param sub_command: the sub-command
         """
-        base = []
         if self._opts["control"] is not None:  # standard use case
-            if self._opts[subcmd] is None:
+            if self._opts[sub_command] is None:
                 base = self._opts["control"].split()
-                base.append(subcmd)
+                base.append(sub_command)
             else:
-                base = self._opts[subcmd].split()
+                base = self._opts[sub_command].split()
         else:  # other use case
-            base = self._opts[subcmd].split()
+            base = self._opts[sub_command].split()
         base = [unquote(token) for token in base]
         return base
 
     def __execute(self, cmd):
-        """ Execute the given command. """
-        env = None
+        """
+        Execute the given command.
+        :param cmd: list containing the command to execute
+        """
         if self._opts["path"] is not None:
-            env = {"PATH": self._opts["path"]}
+            env = {"PATH": self._opts["path"], }
+        else:
+            env = None
+        cmd_str = " ".join(cmd)
         try:
-            log.LOG.debug("executing %s" % " ".join(cmd))
-            result = utils.timed_process(cmd, self._opts["timeout"], env)
-            log.LOG.debug("%s returned: %s" % (" ".join(cmd), result))
+            LOGGER.debug("executing %s" % (cmd_str, ))
+            result = timed_process(cmd, self._opts["timeout"], env)
+            LOGGER.debug("%s returned: %s" % (cmd_str, result))
             return result
-        except utils.ProcessTimedout:
-            log.LOG.warning("%s timed out %d seconds" %
-                            (" ".join(cmd), self._opts["timeout"]))
-            return (1, "", "timeout")
-        except utils.ProcessError:
+        except ProcessTimedout:
+            LOGGER.warning(
+                "%s timed out %d seconds" % (cmd_str, self._opts["timeout"]))
+            return 1, "", "timeout"
+        except ProcessError:
             error = sys.exc_info()[1]
-            log.LOG.warning("error running %s: %s" %
-                            (" ".join(cmd), error))
-            return (1, "", "%s" % error)
+            LOGGER.warning("error running %s: %s" % (cmd_str, error))
+            return 1, "", "%s" % (error, )
 
-    def adjust(self):
+    def cond_adjust(self, careful=False):
         """
         Start/stop according to expected status.
+
+        :param careful: specify if the action should be careful,
+        which means it will make sure that action was performed
+        successfully and that the service is in the expected status
+        @return: True if adjustment performed, False otherwise
         """
-        log.LOG.debug("adjusting service: %s" % self._opts["name"])
-        self._is_new = False
-        to_verify = False
+        LOGGER.debug("conditional adjust for service: %s" % (self.name, ))
+        changed = None
         (return_code, _, _) = self.status()
-        if return_code == 0:
-            if self._opts["expected"] == "stopped":
-                result = self.stop()
-                log.LOG.info("%s stopped with %s" %
-                             (self._opts["name"], result))
-                self._status_log("stop", result)
-                to_verify = True
-        elif return_code == 3:
-            if self._opts["expected"] == "running":
-                result = self.start()
-                log.LOG.info("%s started with %s" %
-                             (self._opts["name"], result))
-                self._status_log("start", result)
-                to_verify = True
-        else:  # unknown/dead/hang...
+        result = (0, "", "")
+        if return_code == 0 and (not self.is_enabled()):
+            result = self.stop()
+            LOGGER.info(
+                "service %s stopped with result: %s" %
+                (self.name, result))
+            self._status_log("stop", result)
+            changed = "stop"
+        elif return_code == 3 and self._opts["expected"] == "running":
+            result = self.start()
+            LOGGER.info(
+                "service %s started with result: %s" %
+                (self.name, result))
+            self._status_log("start", result)
+            changed = "start"
+        elif return_code not in [0, 3]:  # unknown/dead/hang...
             stop_result = self.stop()
             self._status_log("stop", stop_result)
-            log.LOG.info("%s stopped for cleaning %s" %
-                         (self._opts["name"], stop_result))
+            LOGGER.info(
+                "service %s stopped for cleaning with result: %s" %
+                (self.name, stop_result))
             if self._opts["expected"] == "running":
                 result = self.start()
-                log.LOG.info("%s started with %s" %
-                             (self._opts["name"], result))
+                LOGGER.info(
+                    "service %s started with result: %s" %
+                    (self.name, result))
                 self._status_log("start", result)
-            to_verify = True
-        if to_verify:
+            changed = "stop+start"
+        if changed and result[0] != 0:
+            error_message = "error during service %s action %s: %s" % \
+                            (self.name, changed, result, )
+            LOGGER.error(error_message)
+            raise ServiceError(error_message, result)
+        if changed and careful:  # let's do it carefully
+            t_max = time.time() + self._opts["timeout"]
+            check_status = (True, "")
+            while time.time() <= t_max:
+                check_status = self.check()
+                if check_status[0]:
+                    return changed
+                time.sleep(0.2)
+            error_message = "error adjusting service %s, " \
+                            "have been waiting %s" % \
+                            (self.name, self._opts["timeout"])
+            LOGGER.error(error_message)
+            raise ServiceError(
+                error_message, (1, check_status[1], ""))
+        return changed
+
+    def cond_start(self, careful=False):
+        """
+        Conditional start based on status.
+
+        :param careful: specify if the action should be careful,
+        which means it will make sure that action was performed
+        successfully and that the service is in the expected status
+        """
+        LOGGER.debug("conditional start for service: %s" % (self.name, ))
+        changed = None
+        (return_code, _, _) = self.status()
+        result = (0, "", "")
+        if return_code == 3 and self._opts["expected"] == "running":
+            result = self.start()
+            LOGGER.info(
+                "service %s started with result: %s" % (self.name, result))
+            self._status_log("start", result)
+            changed = "start"
+        elif return_code != 0 and self._opts["expected"] == "running":
+            # unknown/dead/hang...
+            stop_result = self.stop()
+            self._status_log("stop", stop_result)
+            LOGGER.info(
+                "service %s stopped for cleaning with result: %s" %
+                (self.name, stop_result))
+            result = self.start()
+            LOGGER.info(
+                "service %s started with result: %s" % (self.name, result))
+            self._status_log("start", result)
+            changed = "stop+start"
+        if changed and result[0] != 0:
+            error_message = "error during service %s action %s: %s" % \
+                (self.name, changed, result, )
+            LOGGER.error(error_message)
+            raise ServiceError(error_message, result)
+        if changed and careful:
             t_max = time.time() + self._opts["timeout"]
             while time.time() <= t_max:
-                checked_status, _ = self.check()
-                if checked_status:
-                    return
+                result = self.status()
+                if result[0] == 0:
+                    return changed
                 time.sleep(0.2)
-            log.LOG.error(
-                "service %s could not be adjusted." % (self._opts["name"], ))
-            raise SimplevisorError("service %s could not be adjusted." %
-                                   (self._opts["name"], ))
+            error_message = "error starting service %s, " \
+                            "have been waiting %s" % \
+                            (self.name, self._opts["timeout"])
+            LOGGER.error(error_message)
+            raise ServiceError(error_message)
+        return changed
+
+    def cond_stop(self, careful=False):
+        """
+        Conditional stop based on status.
+
+        :param careful: specify if the action should be careful,
+        which means it will make sure that action was performed
+        successfully and that the service is in the expected status
+        """
+        LOGGER.debug("conditional stop for service: %s" % (self.name, ))
+        changed = None
+        (return_code, _, _) = self.status()
+        result = (0, "", "")
+        if return_code == 0:
+            result = self.stop()
+            LOGGER.info(
+                "service %s found running, stopped with result: %s" %
+                (self.name, result))
+            self._status_log("stop", result)
+            changed = "stop"
+        elif return_code != 3:  # unknown/dead/hang...
+            result = self.stop()
+            self._status_log("stop", result)
+            LOGGER.info(
+                "service %s found in dirty state: %s, stopped for cleaning"
+                "with result: %s" %
+                (self.name, return_code, result))
+            changed = "stop"
+        if changed and result[0] != 0:
+            error_message = "error during service %s action %s: %s" % \
+                            (self.name, changed, result, )
+            LOGGER.error(error_message)
+            raise ServiceError(error_message, result)
+        if changed and careful:
+            t_max = time.time() + self._opts["timeout"]
+            while time.time() <= t_max:
+                result = self.status()
+                if result[0] == 3:
+                    return changed
+                time.sleep(0.2)
+            error_message = "error stopping service %s, expected return" \
+                "code: 3, received: %s" % (self.name, result[0])
+            LOGGER.error(error_message)
+            raise ServiceError(error_message, result)
+        return changed
 
     def start(self):
         """
         This method takes care of starting the service using the
-        start command. If no start command is provided...
+        start command.
         """
         result = self.__execute(self.get_cmd("start"))
         self._status_log("start", result)
@@ -398,14 +522,14 @@ class Service(object):
             pid_info = self.pidof()
             if pid_info is None:
                 result = (0, "", "")
-                log.LOG.info("%s already stopped" %
-                             (self._opts["name"], ))
+                LOGGER.info(
+                    "service %s already stopped" % (self.name, ))
             else:
                 pids = [x[0] for x in pid_info]
-                utils.kill_pids(pids, self._opts["timeout"])
-                log.LOG.info("%s killed by killing processes: %s" %
-                             (self._opts["name"],
-                             " ".join([str(p) for p in pids])))
+                kill_pids(pids, self._opts["timeout"])
+                LOGGER.info(
+                    "%s killed by killing processes: %s" %
+                    (self.name, " ".join([str(p) for p in pids])))
                 result = (0, "", "")
         else:
             result = self.__execute(self.get_cmd("stop"))
@@ -428,10 +552,10 @@ class Service(object):
         if self._opts["control"] is None and self._opts["status"] is None:
             pid_info = self.pidof()
             if pid_info is None:
-                log.LOG.debug("%s not running" % (self._opts["name"], ))
+                LOGGER.debug("%s not running" % (self.name, ))
                 result = (3, "", "")
             else:
-                log.LOG.debug("%s running" % (self._opts["name"], ))
+                LOGGER.debug("%s running" % (self.name, ))
                 result = (0, "", "")
         else:
             result = self.__execute(self.get_cmd("status"))
@@ -444,7 +568,6 @@ class Service(object):
         """
         (status, _, _) = self.status()
         check_status = True
-        output = "OK"
         if self.is_enabled():
             if status == 0:
                 output = "%s: OK, running, as expected" % (self.name, )
@@ -468,7 +591,7 @@ class Service(object):
                 check_status = False
                 output = "%s: WARNING, in \"dirty\" state: %d" % \
                          (self.name, status)
-        return (check_status, [output, ])
+        return check_status, [output, ]
 
     def restart(self):
         """
@@ -490,22 +613,23 @@ class Service(object):
 
         if restart_cmd:
             result = self.__execute(self.get_cmd("restart"))
-            log.LOG.info("%s restarted with %s" % (self.name, result))
+            LOGGER.info(
+                "service %s restarted with result: %s" % (self.name, result))
             self._status_log("restart", result)
-            return result
         else:
-            rstop = self.stop()
-            rstart = self.start()
-            rjoint = utils.merge_status(rstop, rstart)
-            log.LOG.info("%s stop+start with %s" % (self.name, rjoint))
-            self._status_log("stop+start", rjoint)
-            return rjoint
+            stop_result = self.stop()
+            start_result = self.start()
+            result = merge_status(stop_result, start_result)
+            LOGGER.info(
+                "service %s stop+start result: %s" % (self.name, result))
+            self._status_log("stop+start", result)
+        return result
 
     def pidof(self):
         """ Return the pid of the service. """
         pat_re = self._opts.get("pattern_re", None)
         if pat_re is not None:
-            return utils.pidof(pat_re)
+            return pidof(pat_re)
         return None
 
     def _status_log(self, operation, output):
@@ -516,7 +640,7 @@ class Service(object):
         self._status["log"].append({"time": time.time(),
                                     "operation": operation,
                                     "output": output})
-        self._status["log"] = self._status["log"][-MAXIMUM_LOG:]
+        self._status["log"] = self._status["log"][-MAX_LOG_MESSAGES:]
 
     def is_enabled(self):
         """
@@ -529,16 +653,17 @@ class Service(object):
         """
         Return the string representation.
         """
-        return "service %s" % (self._opts["name"], )
+        return "service %s" % (self.name, )
 
     def get_id(self):
         """
         Return the id of the service.
+        :rtype : str the string representing the service
         """
-        text_id = "%s|%s|%s" % (self._opts["name"],
+        text_id = "%s|%s|%s" % (self.name,
                                 self._opts["expected"],
-                                " ".join(self.get_cmd("start")),)
-        return utils.md5_hash(text_id).hexdigest()
+                                " ".join(self.get_cmd("start")), )
+        return md5_hash(text_id.encode()).hexdigest()
 
     def load_status(self, status):
         """
@@ -553,7 +678,7 @@ class Service(object):
         """
         Return the status to be saved for future runs.
         """
-        # removing log for now
+        # removing log for the time being
         if "log" in self._status:
             del(self._status["log"])
         return self._status
