@@ -14,7 +14,6 @@ limitations under the License.
 
 Copyright (C) 2013 CERN
 """
-from simplevisor.errors import SimplevisorError
 from simplevisor.Simplevisor import Simplevisor
 import unittest
 
@@ -35,14 +34,8 @@ class SimplevisorTest(unittest.TestCase):
             if shouldpass:
                 Simplevisor(**options)
                 continue
-            # else
-            try:
-                Simplevisor(**options)
-                self.fail(
-                    "exception should have been raised for:\nSimplevisor(%s)" %
-                    options)
-            except SimplevisorError:
-                pass
+            else:
+                self.assertRaises(Exception, Simplevisor, **options)
         print("...Simplevisor init ok")
 
 if __name__ == "__main__":
